@@ -16,10 +16,12 @@ Currently supports only version 1.1.1 of the WMS protocol.
 """
 
 import cgi
+import urllib2
 from urllib import urlencode
 from etree import etree
-from .util import openURL
-
+from .util import openURL, testXMLValue
+from fgdc import Metadata
+from iso import MD_Metadata
 
 class ServiceException(Exception):
     """WMS ServiceException
@@ -56,7 +58,7 @@ class WebMapService(object):
 
     
     def __init__(self, url, version='1.1.1', xml=None, 
-                username=None, password=None
+                username=None, password=None, parse_remote_metadata=False
                 ):
         """Initialize."""
         self.url = url
@@ -69,15 +71,19 @@ class WebMapService(object):
         reader = WMSCapabilitiesReader(
                 self.version, url=self.url, un=self.username, pw=self.password
                 )
-        if xml:
-            #read from stored xml
+        if xml:  # read from stored xml
             self._capabilities = reader.readString(xml)
-        else:
-            #read from server
+        else:  # read from server
             self._capabilities = reader.read(self.url)
 
-        #build metadata objects
-        self._buildMetadata()
+        # avoid building capabilities metadata if the response is a ServiceExceptionReport
+        se = self._capabilities.find('ServiceException') 
+        if se is not None: 
+            err_message = str(se.text).strip() 
+            raise ServiceException(err_message, xml) 
+
+        # build metadata objects
+        self._buildMetadata(parse_remote_metadata)
 
     def _getcapproperty(self):
         if not self._capabilities:
@@ -87,7 +93,7 @@ class WebMapService(object):
             self._capabilities = ServiceMetadata(reader.read(self.url))
         return self._capabilities
 
-    def _buildMetadata(self):         
+    def _buildMetadata(self, parse_remote_metadata=False):
         ''' set up capabilities metadata objects '''
         
         #serviceIdentification metadata
@@ -111,7 +117,7 @@ class WebMapService(object):
         #To the WebMapService.contents store only metadata of named layers.
         def gather_layers(parent_elem, parent_metadata):
             for index, elem in enumerate(parent_elem.findall('Layer')):
-                cm = ContentMetadata(elem, parent=parent_metadata, index=index+1)
+                cm = ContentMetadata(elem, parent=parent_metadata, index=index+1, parse_remote_metadata=parse_remote_metadata)
                 if cm.id:
                     if cm.id in self.contents:
                         raise KeyError('Content metadata for layer "%s" already exists' % cm.id)
@@ -239,7 +245,7 @@ class WebMapService(object):
         
     def getServiceXML(self):
         xml = None
-        if self._capabilities:
+        if self._capabilities is not None:
             xml = etree.tostring(self._capabilities)
         return xml
 
@@ -258,22 +264,14 @@ class ServiceIdentification(object):
     
     def __init__(self, infoset, version):
         self._root=infoset
-        self.type = self._root.find('Name').text
+        self.type = testXMLValue(self._root.find('Name'))
         self.version = version
-        self.title = self._root.find('Title').text
-        abstract = self._root.find('Abstract')
-        if abstract is not None:
-            self.abstract = abstract.text
-        else:
-            self.abstract = None
+        self.title = testXMLValue(self._root.find('Title'))
+        self.abstract = testXMLValue(self._root.find('Abstract'))
         self.keywords = [f.text for f in self._root.findall('KeywordList/Keyword')]
-        accessconstraints=self._root.find('AccessConstraints')
-        if accessconstraints is not None:
-            self.accessconstraints = accessconstraints.text
-        fees = self._root.find('Fees')
-        if fees is not None:
-            self.fees = fees.text
-             
+        self.accessconstraints = testXMLValue(self._root.find('AccessConstraints'))
+        self.fees = testXMLValue(self._root.find('Fees'))
+
 class ServiceProvider(object):
     ''' Implements IServiceProviderMetatdata '''
     def __init__(self, infoset):
@@ -313,7 +311,7 @@ class ContentMetadata:
 
     Implements IContentMetadata.
     """
-    def __init__(self, elem, parent=None, index=0):
+    def __init__(self, elem, parent=None, index=0, parse_remote_metadata=False):
         if elem.tag != 'Layer':
             raise ValueError('%s should be a Layer' % (elem,))
         
@@ -323,12 +321,24 @@ class ContentMetadata:
         else:
             self.index = str(index)
         
+        self.id = self.name = testXMLValue(elem.find('Name'))
+
+        # layer attributes
+        self.queryable = int(elem.attrib.get('queryable', 0))
+        self.cascaded = int(elem.attrib.get('cascaded', 0))
+        self.opaque = int(elem.attrib.get('opaque', 0))
+        self.noSubsets = int(elem.attrib.get('noSubsets', 0))
+        self.fixedWidth = int(elem.attrib.get('fixedWidth', 0))
+        self.fixedHeight = int(elem.attrib.get('fixedHeight', 0))
+
         # title is mandatory property
-        self.title = elem.find('Title').text.strip()
-        name = elem.find('Name')
-        self.name = name.text.strip() if name is not None else None
+        self.title = None
+        title = testXMLValue(elem.find('Title'))
+        if title is not None:
+            self.title = title.strip()
+
+        self.abstract = testXMLValue(elem.find('Abstract'))
         
-        self.id=self.name #conform to new interface
         # bboxes
         b = elem.find('BoundingBox')
         self.boundingBox = None
@@ -349,10 +359,10 @@ class ContentMetadata:
                 self.boundingBox = self.parent.boundingBox
 
         # ScaleHint 
- 	sh = elem.find('ScaleHint') 
- 	self.scaleHint = None 
- 	if sh is not None: 
- 	    self.scaleHint = {'min': sh.attrib['min'], 'max': sh.attrib['max']} 
+        sh = elem.find('ScaleHint') 
+        self.scaleHint = None 
+        if sh is not None: 
+            self.scaleHint = {'min': sh.attrib['min'], 'max': sh.attrib['max']} 
 
         attribution = elem.find('Attribution')
         if attribution is not None:
@@ -406,7 +416,10 @@ class ContentMetadata:
             #raise ValueError('%s no SRS available!?' % (elem,))
             #Comment by D Lowe.
             #Do not raise ValueError as it is possible that a layer is purely a parent layer and does not have SRS specified. Instead set crsOptions to None
-            self.crsOptions=None
+            # Comment by Jachym:
+            # Do not set it to None, but to [], which will make the code
+            # work further. Fixed by anthonybaxter
+            self.crsOptions=[]
             
         #Styles
         self.styles = {}
@@ -435,18 +448,41 @@ class ContentMetadata:
         self.timepositions=None
         for extent in elem.findall('Extent'):
             if extent.attrib.get("name").lower() =='time':
-                self.timepositions=extent.text.split(',')
-                break
+                if extent.text:
+                    self.timepositions=extent.text.split(',')
+                    break
 
         # MetadataURLs
         self.metadataUrls = []
         for m in elem.findall('MetadataURL'):
             metadataUrl = {
-                'type': m.attrib['type'],
+                'type': testXMLValue(m.attrib['type'], attrib=True),
+                'format': testXMLValue(m.find('Format')),
+                'url': testXMLValue(m.find('OnlineResource').attrib['{http://www.w3.org/1999/xlink}href'], attrib=True)
+            }
+
+            if metadataUrl['url'] is not None and parse_remote_metadata:  # download URL
+                try:
+                    content = urllib2.urlopen(metadataUrl['url'])
+                    doc = etree.parse(content)
+                    if metadataUrl['type'] is not None:
+                        if metadataUrl['type'] == 'FGDC':
+                            metadataUrl['metadata'] = Metadata(doc)
+                        if metadataUrl['type'] == 'TC211':
+                            metadataUrl['metadata'] = MD_Metadata(doc)
+                except Exception, err:
+                    metadataUrl['metadata'] = None
+
+            self.metadataUrls.append(metadataUrl)
+
+        # DataURLs
+        self.dataUrls = []
+        for m in elem.findall('DataURL'):
+            dataUrl = {
                 'format': m.find('Format').text.strip(),
                 'url': m.find('OnlineResource').attrib['{http://www.w3.org/1999/xlink}href']
             }
-            self.metadataUrls.append(metadataUrl)
+            self.dataUrls.append(dataUrl)
                 
         self.layers = []
         for child in elem.findall('Layer'):
